@@ -1,35 +1,57 @@
-import copy
-import datetime
-import glob
+#TODO:
+from concurrent.futures import ThreadPoolExecutor, as_completed
+from copy import deepcopy
+from datetime import datetime
+from glob import glob
 import gzip
-import fastnumbers
+from fastnumbers import isint, isfloat, fast_int, fast_float
+#TODO:
 from itertools import chain
 from joblib import Parallel, delayed
-import math
-import mmap
-import msgspec
-import operator
-from operator import itemgetter
-import os
-import re
-import shutil
+from math import ceil, log
+from mmap import mmap, PROT_READ
+from msgspec import msgpack
+from operator import eq, ge, gt, le, lt, ne, itemgetter
+from os import makedirs, path
+from re import compile
+from shutil import rmtree
 import sys
-import tempfile
-import zstandard
+from tempfile import mkdtemp
+from zstandard import ZstdCompressor, ZstdDecompressor
+
+# import concurrent.futures
+# import copy
+# import datetime
+# import glob
+# import gzip
+# import fastnumbers
+# #TODO:
+# from itertools import chain
+# from joblib import Parallel, delayed
+# import math
+# import mmap
+# import msgspec
+# import operator
+# from operator import itemgetter
+# import os
+# import re
+# import shutil
+# import sys
+# import tempfile
+# import zstandard
 
 # We use these dictionaries so that when we store the file map, it takes less space on disk.
-FILE_KEY_ABBREVIATIONS = {"data": 1, "cc": 2, "mccl": 3, "ll": 4, "ct": 5, "mctl": 6, "cmpr": 7, "cndata": 8, "cncc": 9, "cnmccl": 10, "cnll": 11}
+FILE_KEY_ABBREVIATIONS_STATS = {"mccl": 3, "mctl": 6, "cnmccl": 10, "cnll": 11}
+FILE_KEY_ABBREVIATIONS_OTHER = {"ll": 4, "cmpr": 7}
+FILE_KEY_ABBREVIATIONS_NOCACHE = {"data": 1, "cc": 2, "ct": 5, "cndata": 8, "cncc": 9}
 
 def open_read_file(file_path, file_extension=""):
     the_file = open(file_path + file_extension, 'rb')
-    return mmap.mmap(the_file.fileno(), 0, prot=mmap.PROT_READ)
+    return mmap(the_file.fileno(), 0, prot=PROT_READ)
 
 def read_str_from_file(file_path, file_extension=""):
     with open(file_path + file_extension, 'rb') as the_file:
-        return the_file.read().rstrip()
-
-def read_int_from_file(file_path, file_extension=""):
-    return fastnumbers.fast_int(read_str_from_file(file_path, file_extension))
+        return the_file.read()
 
 def write_str_to_file(file_path, the_string):
     with open(file_path, 'wb') as the_file:
@@ -63,7 +85,7 @@ def format_column_items(the_list, max_value_length, suffix=""):
 
 def print_message(message, verbose=False):
     if verbose:
-        print(f"{message} - {datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S.%f')}")
+        print(f"{message} - {datetime.now().strftime('%d/%m/%Y %H:%M:%S.%f')}")
 
 def decode_string(x):
     return x.decode()
@@ -77,9 +99,9 @@ def get_conversion_function(column_type):
     elif column_type == "s":
         return decode_string
     elif column_type == "f":
-        return fastnumbers.fast_float
+        return fast_float
     else:
-        return fastnumbers.fast_int
+        return fast_int
 
 def sort_first_column(list_of_lists):
     list_of_lists.sort(key=itemgetter(0))
@@ -110,7 +132,7 @@ def compress_using_2_grams(value, compression_dict):
     return compressed_value
 
 def get_bigram_size(num_bigrams):
-    return math.ceil(math.log(num_bigrams, 2) / 8)
+    return ceil(log(num_bigrams, 2) / 8)
 
 def decompress(compressed_value, compression_dict, bigram_size):
     if compression_dict["compression_type"] == b"c":
@@ -124,31 +146,24 @@ def decompress(compressed_value, compression_dict, bigram_size):
 
     return value
 
-def get_decompressor(decompression_type, decompressor):
-    if not decompression_type:
-        return None
-
-    # We have to instantiate this object more than once because some functions are invoked in parallel, and they cannot be serialized.
-    return zstandard.ZstdDecompressor() if decompression_type == "zstd" else decompressor
-
 def convert_bytes_to_int(b):
     return int.from_bytes(b, byteorder="big")
 
 def serialize(obj):
     #https://github.com/TkTech/json_benchmark
-    return msgspec.msgpack.encode(obj)
+    return msgpack.encode(obj)
 #    return msgpack.packb(obj)
 
 def deserialize(msg):
-    return msgspec.msgpack.decode(msg)
+    return msgpack.decode(msg)
 #    return msgpack.unpackb(msg, strict_map_key=False)
 
 def prepare_tmp_dirs(tmp_dir_path):
     # Figure out where temp files will be stored and create directory, if needed.
     if tmp_dir_path:
-        os.makedirs(tmp_dir_path, exist_ok=True)
+        makedirs(tmp_dir_path, exist_ok=True)
     else:
-        tmp_dir_path = tempfile.mkdtemp()
+        tmp_dir_path = mkdtemp()
 
     tmp_dir_path = fix_dir_path_ending(tmp_dir_path)
 
@@ -156,9 +171,9 @@ def prepare_tmp_dirs(tmp_dir_path):
     tmp_dir_path_outputs = f"{tmp_dir_path}outputs/"
     tmp_dir_path_indexes = f"{tmp_dir_path}indexes/"
 
-    os.makedirs(tmp_dir_path_chunks, exist_ok=True)
-    os.makedirs(tmp_dir_path_outputs, exist_ok=True)
-    os.makedirs(tmp_dir_path_indexes, exist_ok=True)
+    makedirs(tmp_dir_path_chunks, exist_ok=True)
+    makedirs(tmp_dir_path_outputs, exist_ok=True)
+    makedirs(tmp_dir_path_indexes, exist_ok=True)
 
     return tmp_dir_path_chunks, tmp_dir_path_outputs, tmp_dir_path_indexes
 
@@ -169,7 +184,7 @@ def remove_tmp_dir(tmp_dir_path):
     # Remove the temp directory if it was generated by the code (not the user).
     if tmp_dir_path:
         try:
-            shutil.rmtree(tmp_dir_path)
+            rmtree(tmp_dir_path)
             print_message(f"Removed {tmp_dir_path} directory")
         except Exception as e:
             # Don't throw an exception if we can't delete the directory.
@@ -177,43 +192,48 @@ def remove_tmp_dir(tmp_dir_path):
             print(e)
             pass
 
-def combine_into_single_file(tmp_dir_path_chunks, tmp_dir_path_outputs, f4_file_path, num_processes=1, num_rows_per_write=1):
+def combine_into_single_file(tmp_dir_path_chunks, tmp_dir_path_outputs, f4_file_path, num_threads=1, num_rows_per_write=1):
     def _create_file_map(start_end_positions):
         start_end_dict = {}
 
         for x in start_end_positions:
             file_name = x[0]
-            start_end_dict[FILE_KEY_ABBREVIATIONS[file_name]] = x[1:]
-            #start_end_dict[file_name] = x[1:]
+
+            if file_name in FILE_KEY_ABBREVIATIONS_STATS:
+                start_end_dict[FILE_KEY_ABBREVIATIONS_STATS[file_name]] = x[1:]
+            elif file_name in FILE_KEY_ABBREVIATIONS_OTHER:
+                start_end_dict[FILE_KEY_ABBREVIATIONS_OTHER[file_name]] = x[1:]
+            else:
+                start_end_dict[FILE_KEY_ABBREVIATIONS_NOCACHE[file_name]] = x[1:]
 
         serialized = serialize(start_end_dict)
 
         return str(len(serialized)).encode() + b"\n" + serialized
 
     # Find out which files we have
-    data_files = [x for x in glob.glob(f"{tmp_dir_path_chunks}*")]
-    other_files = [x for x in glob.glob(f"{tmp_dir_path_outputs}*")]
+    data_files = [x for x in glob(f"{tmp_dir_path_chunks}*")]
+    other_files = [x for x in glob(f"{tmp_dir_path_outputs}*")]
 
     # Find out how much data we have
-    data_size = sum([os.path.getsize(f) for f in data_files])
+    data_size = sum([path.getsize(f) for f in data_files])
 
     # Determine start and end position of each file within the combined file
     start_end_positions = [["data", 0, data_size]]
     for f in other_files:
         start = start_end_positions[-1][-1]
-        end = start + os.path.getsize(f)
-        start_end_positions.append([os.path.basename(f), start, end])
+        end = start + path.getsize(f)
+        start_end_positions.append([path.basename(f), start, end])
 
     # Adjust the start and end positions based on the size of the values in this list.
     # The size of the file map can change as we advance the positions of the other file contents.
-    original_start_end_positions = copy.deepcopy(start_end_positions)
+    original_start_end_positions = deepcopy(start_end_positions)
     file_map_serialized = _create_file_map(start_end_positions)
     previous_file_map_length = 0
 
     while previous_file_map_length != (len(file_map_serialized)):
         previous_file_map_length = len(file_map_serialized)
 
-        start_end_positions = copy.deepcopy(original_start_end_positions)
+        start_end_positions = deepcopy(original_start_end_positions)
         for i in range(len(start_end_positions)):
             start_end_positions[i][1] += previous_file_map_length
             start_end_positions[i][2] += previous_file_map_length
@@ -224,18 +244,18 @@ def combine_into_single_file(tmp_dir_path_chunks, tmp_dir_path_outputs, f4_file_
     with open(f4_file_path, "wb") as f4_file:
         f4_file.write(file_map_serialized)
 
-        _add_data_chunks(tmp_dir_path_chunks, f4_file, num_processes, num_rows_per_write)
+        _add_data_chunks(tmp_dir_path_chunks, f4_file, num_threads, num_rows_per_write)
 
         for other_file_path in other_files:
             f4_file.write(read_str_from_file(other_file_path))
 
-def _add_data_chunks(tmp_dir_path_chunks, out_file_handle, num_processes, num_rows_per_write):
+def _add_data_chunks(tmp_dir_path_chunks, out_file_handle, num_threads, num_rows_per_write):
     out_lines = []
 
-    for i in range(num_processes):
+    for i in range(num_threads):
         chunk_file_path = f"{tmp_dir_path_chunks}{i}"
 
-        if not os.path.exists(chunk_file_path):
+        if not path.exists(chunk_file_path):
             continue
 
         with open(chunk_file_path, "rb") as chunk_file:
